/**
 * 他モデルの項目参照型詳細設定ダイアログ
 */
define([
  "dojo/_base/array",
  "dojo/_base/lang",
  "dojo/_base/declare",
  "dojo/dom",
  "dojo/dom-style",
  "dojo/store/Memory",
  "dijit/layout/ContentPane",
  "wb/Dialog",
  "wb/ModelitemDetailWidget",
  "wb/ModelitemTypeDetailDialog",
  "wb/ModelReferenceTypeDetailRepository"
], function(
  array,
  lang,
  declare,
  dom,
  domStyle,
  Memory,
  ContentPane,
  Dialog,
  ModelitemDetailWidget,
  ModelitemTypeDetailDialog,
  ModelReferenceTypeDetailRepository
) {
  return declare([ModelitemTypeDetailDialog, ModelReferenceTypeDetailRepository], {
    constructor: function(initValue) {
      this.systemModelStore = new Memory();
      this.setBackupRepositoryKeys();
    },

    createWidget: function(key) {
      return new ModelitemDetailWidget({
        key: key,
        container: this
      });
    },

    setBackupRepositoryKeys: function() {
      this.backupRepositoryKeys = {
        "model/modelitem/relation_storeref/@sortrule": true, // ソートルール
        "model/modelitem/relation_storeref/@roleorder": true, // ソートオーダー
        "model/modelitem/relation_storeref/@rolepriority": true, // 表示優先度
        "model/modelitem/relation_storeref/@rolevalid": true, // 無効判定
        "model/modelitem/relation_storeref/@restrict": true, // 絞り込み
        "model/modelitem/relation/@editmode": true, // 参照連動の設定方法
        "model/modelitem/relation/@timing": true, // 連動タイミング
        "model/modelitem/relation_storeref/@ui": true, // チェックボックスの選択方法
        "model/modelitem/relation_storeref/@displaynotcheckeditem": true, // 選択されなかった項目名を表示する
        "model/modelitem/relation/@navigation": true, // 参照元トレースリンクを表示する
        "model/modelitem/relation/@revlink": true, // 参照先トレースリンクを表示する
      };
    },

    initWidgets: function() {
      this.inherited(arguments);
      // 参照先モデル
      this.idrefSelector = this.widgets["model/modelitem/relation/@idref2"].widget;
      // 連動する項目名
      this.roleidSelector = this.widgets["model/modelitem/relation_storeref/@roleid_22"].widget;
      // 参照先モデル項目
      this.rolecontentSelector = this.widgets["model/modelitem/relation_storeref/@rolecontent2"].widget;
      // 住所項目
      var addressFieldWidgetItem = this.widgets["refermodel/modelitem/@addressfield"];
      addressFieldWidgetItem.getRepositoryKey = function() {
        // 住所項目の本来のリポジトリキー
        return "model/modelitem/zipcodeinfo/@addressfield";
      };
      // 参照モデルの表示優先度項目
      this.rolePrioritySelector = this.widgets["model/modelitem/relation_storeref/@rolepriority"].widget;
      // 参照モデルの無効判定項目
      this.roleValidSelector = this.widgets["model/modelitem/relation_storeref/@rolevalid"].widget;
      // 参照元トレースリンクを表示する項目
      this.relationNavigation = this.widgets["model/modelitem/relation/@navigation"].widget;
    },

    setContent: function() {
      this.content = new ContentPane({
        href: "wb/modelitemDetailTypeDialog/modelReferenceTypeDialog.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      }, this.contentId);
      this.inherited(arguments);
    },

    onDialogContentLoaded: function() {
      this.inherited(arguments);
      this.setModelTypeEvent();
      // ダイアログ内で型の設定値によって表示・非表示を切り替える設定欄
      this.setDisplaySettingRows();
    },

    setScreenResource: function() {
      this.inherited(arguments);
      modelReferenceOptionTitlePane.set("title", wagbyDesigner.jfcRes.LABEL_MODELREFERENCETYPE_DIALOG_OPTION);
    },

    show: function(modelitem) {
      this.inherited(arguments);
      // 他モデルの項目参照設定の表示
      this.setReferModelWidgets();
      // OK ボタンの押下可・不可の判定
      this.isButtonOKEnable();
    },

    setModelTypeEvent: function() {
      var that = this;
      // 参照先モデルタイプラジオボタン
      // ユーザ、システムの切り替えを行なう
      jfcModelTypeUser.on("click", function() {
        that.idrefSelector.set("value", "");
        that.idrefSelector.set("store", that.userModelStore);
      });
      jfcModelTypeSystem.on("click", function() {
        that.idrefSelector.set("value", "");
        that.idrefSelector.set("store", that.systemModelStore);
      });
    },

    setDisplaySettingRows: function() {
      this.referModelItemSelectDialogRoleIdRow = dom.byId("referModelItemSelectDialogRoleIdRow");
      this.referModelitemEditmodeRow = dom.byId("referModelitemEditmodeRow");
      this.referModelitemTimingRow = dom.byId("referModelitemTimingRow");
      this.referModelitemDisplaynotcheckeditemRow = dom.byId("referModelitemDisplaynotcheckeditemRow");
      this.referModelitemCheckboxSearchRow = dom.byId("referModelitemCheckboxSearchRow");
    },

    setEvent: function() {
      var that = this;
      // 参照先モデル選択リストボックス
      this.idrefSelector.on("Change", this.getIdrefSelectorChangeEventFunc());

      // 参照モデルの内容リストボックス変更時にOKボタンの押下可否を判定する
      this.rolecontentSelector.on("Change", function(value) {
        that.isButtonOKEnable();
        // 参照先項目が郵便番号の場合、住所項目の設定欄を表示する
        that.showAddressfieldSetting(value);
      });
      // 連動する項目名
      this.roleidSelector.on("Change", function() {
        that.isButtonOKEnable();
      });
      // OK ボタン押下時の処理
      this.dialog.okButton.on("click", function() {
        that.setValuesToRepository();
        that.dialog.hide();
      });
      // キャンセルボタン押下時の処理
      this.dialog.cancelButton.on("click", function() {
        that.dialog.hide();
      });
      // ダイアログ非表示時の処理
      this.dialog.on("Hide", function() {
        that.hide();
      });
      // ダイアログキャンセル時の処理
      this.dialog.on("Cancel", function() {
        that.cancelValues();
      });
    },

    showAddressfieldSetting: function(rolecontent) {
      var idref = this.idrefSelector.get("value");
      // 参照先の項目が郵便場号以外、読み込み専用、隠し項目の場合は
      // 住所項目設定を表示しない
      if (!wagbyDesigner.isReferModelitemTypeZipcode(idref, rolecontent) ||
           this.modelitem.isReadOnly() || this.modelitem.isHidden()) {
        this.hideAddressfieldRow();
        return;
      }
      this.showAddressfieldRow();
    },

    // 参照先モデル選択リストボックス変更時の処理を行なう関数を返す
    getIdrefSelectorChangeEventFunc: function() {
      var that = this;
      return function(value) {
        // ダイアログの入力可・不可を切り替える
        that.changeReferModelWidgetsDisabled();
        // 連動する項目名の選択肢を再作成する
        that.refreshRoleid2Selector(value);
        // 参照先モデル項目の選択肢を再作成する
        that.refreshRolecontentSelector(value);
        // 参照先モデル項目の設定値クリア後、値を再設定する
        // 選択肢に値が存在しない場合は、空欄となる
        that.rolecontentSelector.set("value", "");
        that.rolecontentSelector.setValue(that.modelitem.getRolecontent());
        // 表示優先度項目の選択肢を再作成する
        that.refreshRolePrioritySelector(value);
        that.rolePrioritySelector.set("value", "");
        that.rolePrioritySelector.setValue(that.modelitem.getRolePriority());
        // 無効判定項目の選択肢を再作成する
        that.refreshRoleValidSelector(value);
        that.roleValidSelector.set("value", "");
        that.roleValidSelector.setValue(that.modelitem.getRoleValid());
        // 選択肢モデルの場合、参照先モデル項目、表示優先度、無効判定項目をセットする
        that.setDefaultRolePriorityAndValid(value);
        that.isButtonOKEnable();
      };
    },

    setDefaultRolePriorityAndValid: function(idref) {
      // モデル項目の「参照先モデル」設定値と同じ場合は設定を変更しない
      if (this.modelitem.getIdref() === idref) {
        return;
      }
      var model = wagbyDesigner.project.getModel(idref);
      if (!model) {
        return;
      }
      // 選択肢モデルの場合
      if (model.isCollectionModel()) {
        // 参照先モデル項目、表示優先度、無効判定項目をセットする、「参照元トレースリンクを表示する」チェックを外す
        this.rolecontentSelector.set("value", "content");
        this.rolePrioritySelector.set("value", "priority");
        this.roleValidSelector.set("value", "valid");
        this.relationNavigation.set("checked", false);
      } else {
        this.rolePrioritySelector.set("value", "");
        this.roleValidSelector.set("value", "");
        this.relationNavigation.set("checked", true);
      }
    },

    refreshRoleid2Selector: function(idref) {
      if (!idref || idref === "") {
        return;
      }
      // 連動する項目名リストボックス選択肢を更新する
      var repositoryKey = "model/modelitem/relation_storeref/@roleid_22";
      var that = this;
      wagbyDesigner.project.getModelitemSelectList(this.getId(), repositoryKey, function(list) {
        // 自モデル項目から参照先モデルが同一の項目のみ選択肢とする
        that.setRoleid2SelectorStore(repositoryKey, idref, list);
      });
    },

    setRoleid2SelectorStore: function(repositoryKey, idref, list) {
      var widgetItem = this.widgets[repositoryKey];
      widgetItem.widget.get("store").setData([]);
      var store = new Memory({
        'data': list,
      });
      store.remove(this.getName()); // 自項目を除く
      // 参照先、外部キーモデルが同一の項目のみ表示する
      store.query(function(item) {
        var modelitem = wagbyDesigner.activeModel.getModelitemByName(item.id);
        return modelitem.getIdref() === idref || modelitem.getOutsideKey() === idref;
      }).forEach(function(data) {
        widgetItem.widget.get("store").add(data);
      });
      widgetItem.setValue(this.modelitem.getRoleid() || "");
    },

    refreshRolecontentSelector: function(idref) {
      // 参照先モデル項目リストボックス選択肢を更新する
      this.setStoreData("model/modelitem/relation_storeref/@rolecontent2", idref, this.modelitem.getRolecontent());
    },

    refreshRolePrioritySelector: function(idref) {
      // 参照モデルの表示優先度項目
      this.setStoreData("model/modelitem/relation_storeref/@rolepriority", idref, this.modelitem.getRolePriority());
    },

    refreshRoleValidSelector: function(idref) {
      // 参照モデルの無効判定項目
      this.setStoreData("model/modelitem/relation_storeref/@rolevalid", idref, this.modelitem.getRoleValid());
    },

    // 参照モデルの絞り込み項目
    refreshRestrictSelector: function() {
      this.setStoreData("model/modelitem/relation_storeref/@restrict", this.getId(), this.modelitem.getRestrict());
    },

    setStoreData: function(repositoryKey, modelId, value) {
      if (!modelId || modelId === "") {
        return;
      }
      var widgetItem = this.widgets[repositoryKey];
      wagbyDesigner.project.getModelitemSelectList(modelId, repositoryKey, function(list) {
        widgetItem.widget.get("store").setData(list);
        widgetItem.setValue(value);
      });
    },

    backupSettings: function() {
      this.inherited(arguments);
      this.backupOptionSettings();
    },

    // 詳細型の設定値を保存する
    backupTypeSettings: function() {
      this.backupType = this.modelitem.getType();
    },

    backupOptionSettings: function() {
      this.backupOptionValues = {};
      for (var repositoryKey in this.backupRepositoryKeys) {
        this.backupOptionValues[repositoryKey] = this.widgets[repositoryKey].value || "";
      }
    },

    restoreSettings: function() {
      this.inherited(arguments);
      this.restoreOptionSettings();
    },

    // 詳細型の設定値を復元する
    restoreTypeSettings: function() {
      // 型のリストボックスの設定値を復元する
      var typeSimple = wagbyDesigner.typeDetailToTypeSimple[this.backupType];
      this.modelitem.setTypeSimple(typeSimple);
      this.modelitem.widgets["model/modelitem/@typeSimple"].setValue(typeSimple);
      // 詳細型の設定値を復元
      this.modelitem.setType(this.backupType);
      this.modelitem.widgets["model/modelitem/@type"].setValue(this.backupType);
    },

    restoreOptionSettings: function() {
      for (var repositoryKey in this.backupRepositoryKeys) {
        var widgetItem = this.widgets[repositoryKey];
        var value = this.backupOptionValues[repositoryKey];
        if (widgetItem.widgetOptions.type === "checkbox") {
          value = value || false;
        }
        widgetItem.widget.set("value", value);
      }
    },

    // 他モデルの項目参照設定の表示
    setReferModelWidgets: function() {
      // ユーザ、システムのモデル名選択肢を作成する
      var that = this;
      wagbyDesigner.project.getUserModelListStore(function(userModelListStore) {
        that.userModelStore = userModelListStore;
        wagbyDesigner.project.getSystemModelListStore(function(systemModelListStore) {
          that.setSystemModelStoreData(systemModelListStore.data);
          // 設定欄を作成する
          that.initReferModelitemWidgets();
          // 住所項目の表示
          that.showAddressfieldSetting(that.modelitem.getRolecontent());
        });
      });
    },

    setSystemModelStoreData: function(data) {
      this.systemModelStore.setData(lang.clone(data));
      // 他モデル参照項目のシステムの参照先モデルからは jholiday, jnews を除く
      this.systemModelStore.remove("jholiday");
      this.systemModelStore.remove("jnews");
    },

    getIdref: function() {
      if (!this.idrefSelector) {
        return "";
      }
      return this.idrefSelector.get("value");
    },

    // 他のモデル項目の参照の初期設定
    initReferModelitemWidgets: function() {
      // 参照先モデル
      var idref = this.modelitem.getIdref();
      this.setIdrefSelector(idref);

      // 入力タイプ選択ラジオボタン
      this.createReferModelSelectTypeWidgets();

      // 連動する項目名リストボックス作成
      this.refreshRoleid2Selector(idref);

      // 参照先モデル項目リストボックス作成
      this.refreshRolecontentSelector(idref);

      // 参照モデルの表示優先度項目
      this.refreshRolePrioritySelector(idref);

      // 参照モデルの無効判定項目
      this.refreshRoleValidSelector(idref);

      // 参照モデルの絞り込み項目
      this.refreshRestrictSelector();

      // 入力欄の入力可・不可を設定する
      this.changeReferModelWidgetsDisabled();

      // 連動する項目名リストボックスの表示・非表示切り替え
      this.displayReferModelitemSelect(this.getInputTypeValue());

      // 住所項目行を取得しておく
      if (!this.addressFieldRow) {
        this.addressFieldRow = dom.byId("modelItemTypeDetailDialogAddressRow2");
      }
    },

    setIdrefSelector: function(idref) {
      // 参照先モデル
      if (idref === "" || this.userModelStore.get(idref)) {
        // ユーザモデル
        jfcModelTypeUser.set("checked", true);
        this.idrefSelector.set("store", this.userModelStore);
      } else {
        // システムモデル
        jfcModelTypeSystem.set("checked", true);
        this.idrefSelector.set("store", this.systemModelStore);
      }
      // モデル名選択リストボックスの作成
      this.idrefSelector.setValue(idref);
    },

    changeSelectTypeDisabled: function(disabled) {
      array.forEach(this.referModelSelectTypeRadioButtons, function(radio) {
        radio.set("disabled", disabled);
      });
    },

    changeRolecontentSelectorDisabled: function(disabled) {
      this.rolecontentSelector.set("disabled", disabled);
    },

    changeReferModelWidgetsDisabled: function() {
      var disabled = true;
      // 入力タイプラジオボタンを入力不可にする
      this.changeSelectTypeDisabled(true);
      // 参照先モデル項目を入力不可にする
      this.changeRolecontentSelectorDisabled(true);
      // 参照先モデル
      var idref = this.idrefSelector.get("value");
      if (idref === "") {
        return;
      }

      // 参照先モデルが選択されていたら入力タイプラジオボタンを
      // 入力可にする
      this.changeSelectTypeDisabled(false);

      // 入力タイプラジオボタン
      var selectedTypeValue = this.getInputTypeValue();
      if (!selectedTypeValue) {
        return;
      }
      // 入力タイプが選択されていたら参照先モデル項目を入力可にする
      this.changeRolecontentSelectorDisabled(false);
    },

    getInputTypeValue: function() {
      var selectedTypeValue;
      array.forEach(this.referModelSelectTypeRadioButtons, function(radio) {
        if (radio.get("checked")) {
          selectedTypeValue = radio.get("value");
        }
      });
      return selectedTypeValue;
    },

    createReferModelSelectTypeWidgets: function() {
      array.forEach(this.referModelSelectTypeRadioButtons, function(radio) {
        radio.destroy();
      });
      this.referModelSelectTypeRadioButtons =
        this.createRadioButtons("referModelSelectType", this.getReferModelSelectTypeWidgetOptions(wagbyDesigner.jfcRes));

      this.placeRadioButtons(this.referModelSelectTypeRadioButtons, "referModelSelectTypeArea");

      // 入力タイプ選択ラジオボタン初期値設定
      this.setReferModelSelectType();
    },

    setReferModelSelectType: function() {
      var that = this;
      // 入力タイプ選択(リストボックス、ラジオ、チェックボックス、別画面検索、参照連動)
      for (var i=0; i<this.referModelSelectTypeRadioButtons.length; i++) {
        var radio = this.referModelSelectTypeRadioButtons[i];
        if (this.getType() && this.getType().indexOf(radio.value) !== -1) {
          radio.set("checked", true);
        }
        radio.on("Click", function(e) {
          that.changeReferModelWidgetsDisabled();
          that.displayReferModelitemSelect(e.target.value);
          that.isButtonOKEnable();
        });
      }
    },

    // 「連動する項目名」、「参照連動の設定方法」、
    // 「選択されなかった項目名の表示」の表示・非表示設定
    displayReferModelitemSelect: function(value) {
      if (value && value.indexOf("lookup") !== -1) {
        this.showSetting(this.referModelItemSelectDialogRoleIdRow);
        // 連動する項目名リストボックス選択肢を更新する
        this.refreshRoleid2Selector(this.idrefSelector.get("value"));
        // 参照連動の設定方法の表示
        this.showSetting(this.referModelitemEditmodeRow);
        this.showSetting(this.referModelitemTimingRow);
      } else {
        this.hideSetting(this.referModelItemSelectDialogRoleIdRow);
        this.hideSetting(this.referModelitemEditmodeRow);
        this.hideSetting(this.referModelitemTimingRow);
      }

      if (value && (value.indexOf("radiobutton") !== -1 ||
            value.indexOf("checkbox") !== -1)) {
        this.showSetting(this.referModelitemDisplaynotcheckeditemRow);
      } else {
        this.hideSetting(this.referModelitemDisplaynotcheckeditemRow);
      }

      if (value && value.indexOf("checkbox") !== -1) {
        this.showSetting(this.referModelitemCheckboxSearchRow);
      } else {
        this.hideSetting(this.referModelitemCheckboxSearchRow);
      }
    },

    showSetting: function(node) {
      this.changeDisplayAttr(node, "");
    },

    hideSetting: function(node) {
      this.changeDisplayAttr(node, "none");
    },

    changeDisplayAttr: function(node, display) {
      domStyle.set(node, "display", display);
    },

    // ダイアログの設定情報をリポジトリへ保存する
    setValuesToRepository: function() {
      this.setReferModelType();
//    this.clearRoleid2IfNecessary();
    },

    clearRoleid2IfNecessary: function() {
      // 入力タイプに参照連動以外が選択された場合は「連動する項目名」をクリアする
      var type = this.modelitem.widgets["model/modelitem/@type"].widget.get("value");
      if (type !== "reference_lookup") {
        // ダイアログ内の「連動する項目名」設定をクリア
        this.roleidSelector.set("value", "");
        // モデル項目の「連動する項目名」設定(非表示)をクリア
        this.modelitem.widgets["model/modelitem/relation_storeref/@roleid_2"].widget.set("value", "");
        this.modelitem.setRoleid("");
      }
    },

    setReferModelType: function() {
      var referType = this.getCheckedValue("#referModelSelectTypeArea input[name='referModelSelectType']");
      var type = "reference_" + referType;
      this.modelitem.widgets["model/modelitem/@type"].widget.set("value", type);
      // 参照先モデル
      this.modelitem.widgets["model/modelitem/relation/@idref"].widget.set("value",
        this.idrefSelector.get("value"));
      // 連動する項目名
      var roleId = this.modelitem.getRoleid();
      if (!roleId || roleId !== this.roleidSelector.get("value")) {
        this.modelitem.widgets["model/modelitem/relation_storeref/@roleid_2"].widget.setValue("");
      }
      this.modelitem.widgets["model/modelitem/relation_storeref/@roleid_2"].widget.set("value",
        this.roleidSelector.get("value"));
      // 参照先モデル項目
      this.modelitem.widgets["model/modelitem/relation_storeref/@rolecontent"].widget.set("value",
        this.rolecontentSelector.get("value"));
    },

    // OK ボタンの押下可・不可を返す
    isButtonOKEnable: function() {
      this.dialog.okButton.set("disabled", true);
      var inputType;
      // 参照先モデル
      var idref = this.getIdref();
      // 参照先モデル項目
      var rolecontent = this.rolecontentSelector.get("value");
      inputType = this.getCheckedValue("#referModelSelectTypeArea input[name='referModelSelectType']");
      if (idref === "" || rolecontent === "" || !inputType) {
        return;
      }
      if (inputType === "lookup" && this.roleidSelector.get("value") === "") {
        return;
      }
      this.dialog.okButton.set("disabled", false);
    },

    showAddressfieldRow: function() {
      this.setAddressfieldRowDisplayAttr({display: ""});
      // 自モデルの文字列項目を選択肢としてセットする
      this.setAddressfieldOptions("refermodel/modelitem/@addressfield");
    },

    hideAddressfieldRow: function() {
      this.setAddressfieldRowDisplayAttr({display: "none"});
    },

    setAddressfieldRowDisplayAttr: function(arg) {
      domStyle.set(this.addressFieldRow, "display", arg.display);
    },

  });
});
