/**
 * モデル項目クラス
 */
define([
  "dojo/_base/declare",
  "dojo/dom-style",
  "dojo/keys",
  "dijit/form/Button",
  "wb/ModelitemDetail",
  "wb/ModelitemRepository",
  "wb/ModelitemRepositoryItem",
  "wb/ModelitemWidget",
  "wb/RepositoryContainer"
], function(
  declare,
  domStyle,
  keys,
  Button,
  ModelitemDetail,
  ModelitemRepository,
  ModelitemRepositoryItem,
  ModelitemWidget,
  RepositoryContainer
) {
  return declare([RepositoryContainer, ModelitemRepository], {
    constructor: function(initValue) {
      this.id = initValue.id;
      this.parent = initValue.parent;
      this.initRepositoryItem(initValue.data);
      this.detail = new ModelitemDetail({
        id: this.id,
        parent: this,
        data: initValue.data
      });
    },

    initRepositoryItem: function(data) {
      for (var key in this.modelitemRepositoryKeys) {
        this.repositoryItems[key] = new ModelitemRepositoryItem({
          id: this.id,
          key: key,
          value: data[key],
          container: this
        });
      }
      // 行番号
      this.no = this.repositoryItems["model/modelitem/@__linenumber"];
      this.setNo(this.repositoryItems["model/modelitem/@__linenumber"].value);
      // 項目名(日本語)
//    this.label = this.repositoryItems["model/modelitem/@label"];
      // 項目名(日本語 レイアウト情報無し)
      this.label = this.repositoryItems["model/modelitem/@labelWithoutLayout"];
      // 項目名(英語)
      this.name = this.repositoryItems["model/modelitem/@name"];
      // 項目の型(シンプルバージョン)
      var typeSimple = wagbyDesigner.typeDetailToTypeSimple[this.getType()];
      this.typeSimpleRepositoryItem = new ModelitemRepositoryItem({
        id: this.id,
        key: "model/modelitem/@typeSimple",
        value: typeSimple,
        container: this
      });
      this.repositoryItems["model/modelitem/@typeSimple"] = this.typeSimpleRepositoryItem;
      // レイアウト名
      this.layoutName = this.repositoryItems["model/modelitem/@layoutName"];
    },

    confirmSendNameValue: function(widgetItem, modifyMapOtherModel) {
      var isSendNameValue = true;
      var modelitem = widgetItem.container;
      if (!wagbyDesigner.confirmChangeReferencedItemValue(widgetItem, widgetItem.value, modifyMapOtherModel)) {
        // モデル項目ID 変更後にモデル項目行を変更可にする
        this.enableModelitem(modelitem);
        isSendNameValue = false;
      }
      return isSendNameValue;
    },

    setNameWidgetEvent: function(widgetItem) {
      var that = this;
      var disableExceptItems = {"model/modelitem/@name": true};
      widgetItem.widget.on("change", function(value) {
        var modifiedWidgetItem = that.widgets["model/modelitem/@name"];
        var modelitem = modifiedWidgetItem.container;
        var oldName = modelitem.getName();
        // モデル項目ID 変更前にモデル項目行を変更不可にする
        that.disableModelitem(modelitem, disableExceptItems);
        // 他の項目から参照されているか確認を行なう
        var modifyMapOtherModel = wagbyDesigner.checkItemReferenced(modifiedWidgetItem);
        if (modifyMapOtherModel && that.confirmSendNameValue(modifiedWidgetItem, modifyMapOtherModel)) {
          // 変更可の場合、変更した値をサーバへ送信する
          modifiedWidgetItem.sendData(modifiedWidgetItem, value, {onSuccess: function() {
            that.enableModelitem(modelitem, disableExceptItems);
            that.clearContainerLayoutIfNecessary(modifiedWidgetItem.value, value);
            wagbyDesigner.modelPane.modelitemPaneContent.changeStoreModelitemName(oldName, value);
            if (that.isPrimaryKey()) {
              // 主キー項目の項目IDが変更された場合は、関連図のモデルデータ再生成フラグをセットする
              wagbyDesigner.setRelationModelDataNeedsRefresh(true);
            }
          }, onError: function(response) {
            var msg = wagbyDesigner.jfcRes.MSG_ERROR_SAVE_REPOSITORY_DATA;
            if (response.status === 409) {
              msg = wagbyDesigner.jfcRes.MSG_ERROR_SAVE_REPOSITORY_DATA_LOCKED;
            }
            wagbyDesigner.showDialog(msg);
            widgetItem.setValue(oldName);
            that.enableModelitem(modelitem, disableExceptItems);
          }});
        } else {
          that.enableModelitem(modelitem, disableExceptItems);
        }
      });
    },

    // コンテナ項目を通常項目へ変更した場合、コンテナのレイアウト設定をクリアする
    clearContainerLayoutIfNecessary: function(oldValue, newValue) {
      if (!(oldValue.indexOf("/") !== -1 && newValue.indexOf("/") === -1)) {
        return;
      }
      // ラベル名
      this.clearWidgetValue("model/modelitem/@containerLabel");
      // セル結合(行)
      this.clearWidgetValue("model/modelitem/@containerRowspan");
      // セル結合(列)
      this.clearWidgetValue("model/modelitem/@containerColspan");
    },

    // 一覧表示の設定を外した場合、一覧のレイアウト設定をクリアする
    clearListLayoutIfNecessary: function(value) {
      if (value) {
        return;
      }
      // ラベル名
      this.clearWidgetValue("model/modelitem/@showListLabel");
      // セル結合(行)
      this.clearWidgetValue("model/modelitem/@showListRowspan");
      // セル結合(列)
      this.clearWidgetValue("model/modelitem/@showListColspan");
    },

    clearWidgetValue: function(key) {
      var widgetItem = this.widgets[key];
      if (!widgetItem) {
        return;
      }
      widgetItem.widget.set("value", "");
    },

    createWidget: function() {
      for (var key in this.modelitemRepositoryKeys) {
        if (!this.widgets[key]) {
          this.widgets[key] = new ModelitemWidget({
            key: key,
            value: this.repositoryItems[key].value,
            container: this
          });
          // 特別な項目に対する追加設定
          this.setAdditionalRule(key);
        } else if (!this.widgets[key].widget) {
          this.widgets[key].setWidget();
        }
      }
      var labelWidget = this.widgets["model/modelitem/@labelWithoutLayout"].widget;
      if (labelWidget && labelWidget.textbox) {
        domStyle.set(labelWidget.textbox, "imeMode", "active");
      }
      var nameWidget = this.widgets["model/modelitem/@name"].widget;
      if (nameWidget && nameWidget.textbox) {
        domStyle.set(nameWidget.textbox, "imeMode", "disabled");
      }
      // 項目名、項目ID の入力欄をカーソルキーで移動できるようにイベントを設定する
      this.setCursorMoveEnable();
    },

    createWidgetByRepositoryKey: function(repositoryKey) {
      if (this.widgets[repositoryKey]) {
        return;
      }
      var widgetItem = new ModelitemWidget({
        key: repositoryKey,
        value: this.repositoryItems[repositoryKey].value,
        container: this
      });
      this.widgets[repositoryKey] = widgetItem;
      return widgetItem;
    },

    setCursorMoveEnable: function() {
      // 項目名にイベントを設定する
      var that = this;
      var labelWithoutLayoutWidget = this.widgets["model/modelitem/@labelWithoutLayout"].widget;
      labelWithoutLayoutWidget.on("keypress", this.getMoveCursorModelitemFunction(this));
      labelWithoutLayoutWidget.on("change", function() {
        if (that.isPrimaryKey()) {
          // 主キー項目の項目名が変更された場合は、関連図のモデルデータ再生成フラグをセットする
          wagbyDesigner.setRelationModelDataNeedsRefresh(true);
        }
      });
      // 項目ID にイベントを設定する
      this.widgets["model/modelitem/@name"].widget.on("keypress", this.getMoveCursorModelitemFunction(this));
    },

    // カーソル移動の関数を返す
    getMoveCursorModelitemFunction: function(modelitem) {
      return function(event) {
        var isIgnoreCaretPosition = false;
        if (event.charOrCode === keys.ENTER) {
          isIgnoreCaretPosition = true;
        }
        var element = event.srcElement;
        if (event.charOrCode === keys.UP_ARROW ||
            (event.shiftKey === true && event.charOrCode === keys.ENTER)) {
          modelitem.moveCursorPreviousModelitem(element, isIgnoreCaretPosition);
        } else if (event.charOrCode === keys.DOWN_ARROW ||
                   (event.shiftKey === false && event.charOrCode === keys.ENTER)) {
          modelitem.moveCursorNextModelitem(element, isIgnoreCaretPosition);
        } else if (event.charOrCode === keys.LEFT_ARROW) {
          modelitem.moveCursorLabel(element);
        } else if (event.charOrCode === keys.RIGHT_ARROW) {
          modelitem.moveCursorName(element);
        }
      }
    },

    // 前行のモデル項目の入力欄へフォーカスをセットする
    moveCursorPreviousModelitem: function(element, isIgnoreCaretPosition) {
      // 入力欄の先頭にキャレットが無い場合にはカーソル移動させない
      // Enter キー押下の場合は、キャレット位置を確認しない
      if (!isIgnoreCaretPosition && element.selectionEnd !== 0) {
        return;
      }
      // 前行の行番号
      var previousModelitemNo = this.getNo() - 1;
      if (previousModelitemNo === 0) {
        // 先頭行の場合は、最終行の行番号に入れ替える
        previousModelitemNo = this.parent.getModelitemCount();
      }
      var previousModelitem = this.parent.getModelitemByNo(previousModelitemNo);
      if (!previousModelitem) {
        return;
      }
      wagbyDesigner.focusAndSelect(
        previousModelitem.widgets[element.name].widget.textbox
      );
    },

    // 次行のモデル項目の入力欄へフォーカスをセットする
    moveCursorNextModelitem: function(element, isIgnoreCaretPosition) {
      // 入力欄の最後にキャレットが無い場合にはカーソル移動させない
      // Enter キー押下の場合は、キャレット位置を確認しない
      if (!isIgnoreCaretPosition) {
        var length = 0;
        if (element.value) {
          length = element.value.length;
        }
        if (element.selectionStart !== length) {
          return;
        }
      }
      // 次行の行番号
      var nextModelitemNo = this.getNo() + 1;
      if (nextModelitemNo > this.parent.getModelitemCount()) {
        // 最終行の場合は、先頭行の行番号に入れ替える
        nextModelitemNo = 1;
      }
      var nextModelitem = this.parent.getModelitemByNo(nextModelitemNo);
      if (!nextModelitem) {
        return;
      }
      wagbyDesigner.focusAndSelect(
        nextModelitem.widgets[element.name].widget.textbox
      );
    },

    // 項目名にフォーカスをセットする
    moveCursorLabel: function(element) {
      // 項目名入力欄で左カーソルを押下した場合、移動先が無いので何もしない
      if (element.name === "model/modelitem/@labelWithoutLayout") {
        return;
      }
      // 項目ID 入力欄の先頭にキャレットが無い場合にはカーソル移動させない
      if (element.selectionEnd !== 0) {
        return;
      }
      wagbyDesigner.focusAndSelect(
        this.widgets["model/modelitem/@labelWithoutLayout"].widget.textbox
      );
    },

    // 項目ID にフォーカスをセットする
    moveCursorName: function(element) {
      // 項目ID 入力欄で右カーソルを押下した場合、移動先が無いので何もしない
      if (element.name === "model/modelitem/@name") {
        return;
      }
      // 項目名入力欄の最後にキャレットが無い場合にはカーソル移動させない
      var length = 0;
      if (element.value) {
        length = element.value.length;
      }
      if (element.selectionStart !== length) {
        return;
      }
      wagbyDesigner.focusAndSelect(
        this.widgets["model/modelitem/@name"].widget.textbox
      );
    },

    setAdditionalRule: function(key) {
      var that = this;
      if (key === "model/modelitem/@name") {
        // 項目ID 変更時の処理を設定する
        this.setNameWidgetEvent(this.widgets[key]);
      } else if (key === "model/modelitem/@typeSimple") {
        // 項目の型変更時の処理を設定する
        this.widgets[key].widget.on("change", this.typeSimpleOnChngeEventFunction());
      } else if (key === "model/modelitem/@hasConditionModel" ||
                 key === "model/modelitem/@hasListModel") {
        // 一覧のチェックを外した時の処理を設定する
        if (key === "model/modelitem/@hasListModel") {
          this.widgets[key].widget.on("change", function(value) {
            that.clearListLayoutIfNecessary(value);
          });
        }
      }
    },

    // 行番号
    getNo: function() {
      return this.no.value;
    },

    setNo: function(no) {
      this.no.value = parseInt(no, 10);
    },

    // 項目名を返す
    getLabel: function() {
      if (this.label.value && this.label.value !== "") {
        return this.label.value;
      }
      return this.getName();
    },

    getLabelWithName: function() {
      return this.getLabel() + "(" + this.getName() + ")";
    },

    getId: function() {
      return this.parent.getId();
    },

    getName: function() {
      return this.name.value;
    },

    getLayoutName: function() {
      return this.layoutName.value;
    },

    getModelitemId: function() {
      return this.id;
    },

    getTypeSimple: function() {
      return this.typeSimpleRepositoryItem.value;
    },

    setTypeSimple: function(value) {
      this.typeSimpleRepositoryItem.value = value;
    },

    getType: function() {
      if (!this.repositoryItems) {
        return;
      }
      var repositoryItem = this.repositoryItems["model/modelitem/@type"];
      if (!repositoryItem) {
        return "";
      }
      var type = repositoryItem.value;
      if (type) {
        return type;
      }
      return "string";
    },

    setType: function(value) {
      this.repositoryItems["model/modelitem/@type"].value = value;
    },

    getIdref: function() {
      return this.repositoryItems["model/modelitem/relation/@idref"].value;
    },

    setIdref: function(idref) {
      this.repositoryItems["model/modelitem/relation/@idref"].value = idref;
    },

    getRolecontent: function() {
      return this.repositoryItems["model/modelitem/relation_storeref/@rolecontent"].value;
    },

    setRolecontent: function(rolecontent) {
      this.repositoryItems["model/modelitem/relation_storeref/@rolecontent"].value = rolecontent;
    },

    getRoleid: function() {
      return this.repositoryItems["model/modelitem/relation_storeref/@roleid_2"].value;
    },

    setRoleid: function(roleid) {
      this.repositoryItems["model/modelitem/relation_storeref/@roleid_2"].value = roleid;
    },

    getRolePriority: function() {
      return this.repositoryItems["model/modelitem/relation_storeref/@rolepriority"].value;
    },

    setRolePriority: function(rolepriority) {
      this.repositoryItems["model/modelitem/relation_storeref/@rolepriority"].value = rolepriority;
    },

    getRoleOrder: function() {
      return this.repositoryItems["model/modelitem/relation_storeref/@roleorder"].value;
    },

    setRoleOrder: function(roleorder) {
      this.repositoryItems["model/modelitem/relation_storeref/@roleorder"].value = roleorder;
    },

    getRestrict: function() {
      return this.repositoryItems["model/modelitem/relation_storeref/@restrict"].value;
    },

    setRestrict: function(restrict) {
      this.repositoryItems["model/modelitem/relation_storeref/@restrict"].value = restrict;
    },

    getRoleValid: function() {
      return this.repositoryItems["model/modelitem/relation_storeref/@rolevalid"].value;
    },

    setRoleValid: function(rolevalid) {
      this.repositoryItems["model/modelitem/relation_storeref/@rolevalid"].value = rolevalid;
    },

    getAddressfield: function() {
      return this.repositoryItems["model/modelitem/zipcodeinfo/@addressfield"].value;
    },

    getOutsideKey: function() {
      return this.detail.repositoryItems["model/modelitem/@outsideKey"].value;
    },

    getNote: function() {
      return this.detail.repositoryItems["model/modelitem/@note"].value || "";
    },

    getDetailDisplayOrder: function() {
      return this.repositoryItems["model/modelitem/@detailDisplayOrder"].value || "";
    },

    getDetailLabelAlign: function() {
      return this.detail.repositoryItems["presentation/displayitem/@labelalign"].value || "";
    },

    isDetailLabelHidden: function() {
      if (this.detail.repositoryItems["presentation/displayitem/@displaylabel=false"].value === wagbyDesigner.jfcRes.OPTION_CHECKED) {
        return true;
      }
      return false;
    },

    isItemTypeContainer: function() {
      if (this.getTypeSimple() === "container") {
        return true;
      }
      return false;
    },

    isContainerId: function() {
      if (this.getTypeSimple() === "containerId") {
        return true;
      }
      return false;
    },

    isContainerItem: function() {
      if (this.getName() && this.getName().indexOf("/") !== -1) {
        return true;
      }
      return false;
    },

    isMultipleItem: function() {
      if (this.detail.repositoryItems["model/modelitem/@multiple"].value === wagbyDesigner.jfcRes.OPTION_CHECKED) {
        return true;
      }
      return false;
    },

    hasConditionModel: function() {
      return this.repositoryItems["model/modelitem/@hasConditionModel"].value;
    },

    hasListModel: function() {
      return this.repositoryItems["model/modelitem/@hasListModel"].value;
    },

    getColumnForSearch: function() {
      return parseInt(this.repositoryItems["presentation/displayitem/@columnForSearch"].value, 10);
    },

    getTypeForSearch: function() {
      return this.isSelected("presentation/displayitem/@typeForSearch");
    },

    getTypeForShowList: function() {
      return this.isSelected("presentation/displayitem/@typeForShowList=hidden");
    },

    getTypeForShow: function() {
      return this.isSelected("presentation/displayitem/@typeForShow=hidden");
    },

    isSelected: function(key) {
      var repositoryItem = this.detail.repositoryItems[key] || this.repositoryItems[key];
      if (!repositoryItem) {
        return false;
      }
      if (wagbyDesigner.jfcRes.OPTION_CHECKED === repositoryItem.value) {
        return true;
      }
      return false;
    },

    getShowListLabel: function() {
      return this.repositoryItems["model/modelitem/@showListLabel"].value || "";
    },

    setShowListLabel: function(value) {
      this.repositoryItems["model/modelitem/@showListLabel"].value = value;
    },

    getShowListRowspan: function() {
      return this.repositoryItems["model/modelitem/@showListRowspan"].value || "";
    },

    setShowListRowspan: function(value) {
      this.repositoryItems["model/modelitem/@showListRowspan"].value = value;
    },

    getShowListColspan: function() {
      return this.repositoryItems["model/modelitem/@showListColspan"].value || "";
    },

    getContainerLabel: function() {
      return this.repositoryItems["model/modelitem/@containerLabel"].value || "";
    },

    setContainerLabel: function(value) {
      this.repositoryItems["model/modelitem/@containerLabel"].value = value;
    },

    getContainerRowspan: function() {
      var value = parseInt(this.repositoryItems["model/modelitem/@containerRowspan"].value, 10);
      if (isNaN(value)) {
        return "";
      }
      return value;
    },

    setContainerRowspan: function(value) {
      this.repositoryItems["model/modelitem/@containerRowspan"].value = value;
    },

    getContainerColspan: function() {
      var value = parseInt(this.repositoryItems["model/modelitem/@containerColspan"].value, 10);
      if (isNaN(value)) {
        return "";
      }
      return value;
    },

    getWizard: function() {
      return this.detail.repositoryItems["model/modelitem/@wizard"].value || "";
    },

    show: function() {
      for (var key in this.repositoryItems) {
        this.widgets[key].show();
      }
      // 項目の型ダイアログ表示ボタン
      var modelitem = this;
      this.typeDetailDialogButton = new Button({
        id: "typeDetailDialogButton" + modelitem.id,
        name: "typeDetailDialogButton" + modelitem.id,
        iconClass: "dijitIconEdit",
        style: "magin: 0; padding: 0;",
        onClick: function(b) { wagbyDesigner.showModelitemTypeDetailDialog(modelitem); }
      });

      this.detailButton = new Button({
        id: "detailButton" + modelitem.id,
        name: "detailButton" + modelitem.id,
        label: wagbyDesigner.jfcRes.LABEL_BTN_MODELITEM_DETAIL, // "詳細"
        style: "magin: 0; padding: 0;",
        onClick: function(e) { wagbyDesigner.showModelitemDetailDialog(modelitem.id); }
      });

      this.typeDetailDialogButton.placeAt("place_model_modelitem_typeSimple" + this.id);
      this.detailButton.placeAt("place_detailButton" + this.id);

      // コンテナ、コンテナIDの場合、項目型詳細設定ダイアログ表示ボタンを押下不可にする
      if (wagbyDesigner.isModelitemTypeContainer(this.getTypeSimple())) {
        this.typeDetailDialogButton.set("disabled", true);
      }
      // コンテナの場合、検索、一覧チェックボックスを入力不可にする
      if (this.isItemTypeContainer()) {
        this.setConditionListDisabled(true);
      }
    },

    typeSimpleOnChngeEventFunction: function() {
      var modelitem = this;
      return function(typeSimple) {
        modelitem.setTypeSimple(typeSimple);
        // 他のモデルの参照の場合、参照先項目設定ダイアログを開く
        if (typeSimple === "referModel") {
          wagbyDesigner.showModelitemTypeDetailDialog(modelitem);
          return;
        }
        // コンテナ関係の型が選択された場合は、詳細型設定ダイアログ表示ボタンを押下不可にする
        var multipleValue = "";
        if (typeSimple.indexOf("container") !== -1) {
          modelitem.typeDetailDialogButton.set("disabled", true);
        } else {
          modelitem.typeDetailDialogButton.set("disabled", false);
        }
        // 繰り返しコンテナの場合は、検索、一覧チェックボックスを入力不可にする
        if (typeSimple === "container") {
          modelitem.setConditionListDisabled(true);
        } else if (!modelitem.parent.isCollectionModel()) {
          modelitem.setConditionListDisabled(false);
        }
        wagbyDesigner.primitiveTypeDetailDialog.setModelitem(modelitem);
        wagbyDesigner.primitiveTypeDetailDialog.setTypeDetail(typeSimple);
        modelitem.widgets["model/modelitem/@type"].widget.set("value", wagbyDesigner.typeSimpleDefaultDetailType[typeSimple]);
        // 「他モデルの項目参照」以外の型の場合「連動する項目名」の値をクリアしておく
        modelitem.setRoleid("");
      };
    },

    // 検索・一覧チェックボックスの入力可・不可制御
    // 項目の型が「繰り返しコンテナ」の場合にチェックを外して入力不可にする
    setConditionListDisabled: function(disabled) {
      var conditionModelWidgetItem = this.widgets["model/modelitem/@hasConditionModel"];
      var listModelWidgetItem = this.widgets["model/modelitem/@hasListModel"];
      if (!conditionModelWidgetItem || !conditionModelWidgetItem.widget ||
          !listModelWidgetItem || !listModelWidgetItem.widget) {
        return;
      }
      if (disabled) {
        conditionModelWidgetItem.setValue("");
        listModelWidgetItem.setValue("");
      }
      conditionModelWidgetItem.widget.set("disabled", disabled);
      listModelWidgetItem.widget.set("disabled", disabled);
    },

    isPrimaryKey: function() {
      if (this.detail.repositoryItems["model/modelitem/@primaryKey"].value === wagbyDesigner.jfcRes.OPTION_CHECKED) {
        return true;
      }
      return false;
    },

    isForeignKey: function() {
      var value = this.detail.repositoryItems["model/modelitem/@outsideKey"].value;
      if (value && value !== "") {
        return true;
      }
      return false;
    },

    destroy: function() {
      this.detail.destroy();
      this.detail = null;
      this.repositoryItems = null;
    },

    destroyWidget: function() {
      for (var key in this.modelitemRepositoryKeys) {
        if (this.widgets[key] && this.widgets[key].widget) {
          this.widgets[key].widget.destroyRecursive();
          this.widgets[key].widget = null;
          this.widgets[key] = null;
        }
      }
      if (this.typeDetailDialogButton) {
        this.typeDetailDialogButton.destroyRecursive();
        this.typeDetailDialogButton = null;
      }
      if (this.detailButton) {
        this.detailButton.destroyRecursive();
        this.detailButton = null;
      }
    },

    hide: function() {
      this.destroyWidget();
    },

    // モデル項目のリポジトリを更新する
    modifyRepositoryData: function(modifiedModelitemData) {
      for (var repositoryKey in modifiedModelitemData) {
        var value = modifiedModelitemData[repositoryKey];
        if (this.repositoryItems[repositoryKey]) {
          this.repositoryItems[repositoryKey].value = value;
          var widgetItem = this.widgets[repositoryKey];
          if (widgetItem) {
            widgetItem.setValue(this.getWidgetItemValue(repositoryKey, value), false);
          }
        } else {
          // モデル項目詳細のリポジトリを更新する
          var modifiedModelitemDetailData = {};
          modifiedModelitemDetailData[repositoryKey] = value;
          this.detail.modifyRepositoryData(modifiedModelitemDetailData);
        }
      }
      this.setNo(this.repositoryItems["model/modelitem/@__linenumber"].value);
    },

    getWidgetItemValue: function(repositoryKey, value) {
      if (!(repositoryKey.lastIndexOf("model/modelitem/@hasConditionModel", 0) === 0 ||
          repositoryKey.lastIndexOf("model/modelitem/@hasListModel", 0) === 0)) {
        return value;
      }
      if (value && value !== "") {
        return true;
      }
      return false;
    },

    // モデル項目行を編集不可にする
    disableModelitem: function(modelitem, exceptItems) {
      wagbyDesigner.modelPane.modelitemPaneContent.setModelitemDisabled(modelitem, true, exceptItems);
    },

    // モデル項目行を編集可にする
    enableModelitem: function(modelitem, exceptItems) {
      wagbyDesigner.modelPane.modelitemPaneContent.setModelitemDisabled(modelitem, false, exceptItems);
    },

    isReferModel: function() {
      if (this.getTypeSimple() === "referModel") {
        return true;
      }
      return false;
    },

    isReadOnly: function() {
      return (
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@readonly'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@readonly_insert'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@readonly_update'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@readonly_copy'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@readonly_updatelist'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@readonly_updatelistInParent'].value)
      );
    },

    isHidden: function() {
      return (
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_insert'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_update'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_copy'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_updatelist'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_updatelistInParent'].value)
      );
    },

    isAllHidden: function() {
      if (wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden'].value)) {
        return true;
      }
      return wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_insert'].value) &&
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_update'].value) &&
          wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_copy'].value);
    },

    isInsertUpdateCopyHidden: function() {
      return wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden'].value) ||
        wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_insert'].value) ||
          wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_update'].value) ||
            wagbyDesigner.isChecked(this.detail.repositoryItems['presentation/displayitem/@hidden_copy'].value);
    },

    isLayoutItem: function() {
      return this.detail.repositoryItems["model/modelitem/@layout"].value === wagbyDesigner.jfcRes.OPTION_CHECKED;
    },

  });
});
